@@ -21,14 +21,26 @@
 #define Py_RETURN_NONE return Py_INCREF(Py_None), Py_None
 #endif
 
-// PYUSB_STATIC char cvsid[] = "$Id: pyusb.c,v 1.24 2007/05/15 21:20:55 wander Exp $";
+#if (PY_VERSION_HEX < 0x02050000)
+typedef int Py_ssize_t;
+#endif
+
+// PYUSB_STATIC char cvsid[] = "$Id: pyusb.c,v 1.29 2009/04/06 18:03:10 wander Exp $";
 
 /*
  * USBError
  */
 PYUSB_STATIC PyObject *PyExc_USBError;
 
-#define PyUSB_Error() PyErr_SetString(PyExc_USBError, usb_strerror())
+void static PyUSB_Error(void)
+{
+    char *error_message = usb_strerror();
+
+    if (!strcmp(error_message, "No Error"))
+        error_message = "No error message";
+
+    PyErr_SetString(PyExc_USBError, error_message);
+}
 
 #define SUPPORT_NUMBER_PROTOCOL(_Arg) \
 	(PyNumber_Check(_Arg) || PyString_Check(_Arg) || PyUnicode_Check(_Arg))
@@ -126,7 +138,7 @@ PYUSB_STATIC u_int8_t getByte(
  */
 PYUSB_STATIC char *getBuffer(
 	PyObject *obj,
-	int *size
+	Py_ssize_t *size
 	)
 {
 	char *p = NULL;
@@ -451,6 +463,13 @@ PYUSB_STATIC PyMemberDef Py_usb_Interface_Members[] = {
 	 "Similar to bDeviceProtocol in the device\n"
 	 "descriptor, but for devices whose class is defined by the interface."},
 
+ 	{"iInterface",
+ 	 T_UBYTE,
+ 	 offsetof(Py_usb_Interface, iInterface),
+ 	 READONLY,
+ 	 "Similar to iInterface in the device\n"
+ 	 "descriptor, but for devices whose class is defined by the interface."},
+
 	{"endpoints",
 	 T_OBJECT,
 	 offsetof(Py_usb_Interface, endpoints),
@@ -820,6 +839,12 @@ PYUSB_STATIC PyMemberDef Py_usb_Device_Members[] = {
 	 "An index that points to a string containing the\n"
 	 "device's serial number."},
 
+ 	{"devnum",
+ 	 T_UBYTE,
+ 	 offsetof(Py_usb_Device, devnum),
+ 	 RO,
+ 	 "devnum of the device.\n"},
+
 	{NULL}
 };
 
@@ -933,6 +958,7 @@ PYUSB_STATIC void set_Device_fields(
 	device->iSerialNumber = desc->iSerialNumber;
 	strcpy(device->filename, dev->filename);
 	device->dev = dev;
+ 	device->devnum = dev->devnum;
 
 	if (!dev->config) {
 		device->configurations = PyTuple_New(0);
@@ -1104,7 +1130,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_controlMsg(
 	int index = 0;
 	char *bytes;
 	PyObject *data;
-	int size;
+	Py_ssize_t size;
 	int timeout = DEFAULT_TIMEOUT;
 	int ret;
 	int as_read = 0;
@@ -1278,7 +1304,9 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_detachKernelDriver(
 	)
 {
 	int interfaceNumber;
+#ifdef LIBUSB_HAS_DETACH_KERNEL_DRIVER_NP
 	int ret;
+#endif
 	Py_usb_DeviceHandle *_self = (Py_usb_DeviceHandle *) self;
 
 	if (SUPPORT_NUMBER_PROTOCOL(args)) {
@@ -1304,7 +1332,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_detachKernelDriver(
 	ret = usb_detach_kernel_driver_np(_self->deviceHandle, interfaceNumber);
 	Py_END_ALLOW_THREADS
 
-	if (ret) {
+	if (ret < 0) {
 		PyUSB_Error();
 		return NULL;
 	} 
@@ -1326,7 +1354,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_releaseInterface(
 		ret = usb_release_interface(_self->deviceHandle, _self->interfaceClaimed);
 		Py_END_ALLOW_THREADS
 
-		if (ret) {
+		if (ret < 0) {
 			PyUSB_Error();
 			return NULL;
 		} else {
@@ -1386,7 +1414,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_bulkWrite(
 	int endpoint;
 	int timeout = DEFAULT_TIMEOUT;
 	char *data;
-	int size;
+	Py_ssize_t size;
 	PyObject *bytes;
 	int ret;
 	PyObject *retObj;
@@ -1493,7 +1521,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_interruptWrite(
 	int endpoint;
 	int timeout = DEFAULT_TIMEOUT;
 	char *data;
-	int size;
+	Py_ssize_t size;
 	PyObject *bytes;
 	int ret;
 	PyObject *retObj;
@@ -1615,7 +1643,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_resetEndpoint(
 	ret = usb_resetep(_self->deviceHandle, endpoint);
 	Py_END_ALLOW_THREADS
 
-	if (ret) {
+	if (ret < 0) {
 		PyUSB_Error();
 		return NULL;
 	} else {
@@ -1634,7 +1662,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_reset(
 	ret = usb_reset(((Py_usb_DeviceHandle *) self)->deviceHandle);
 	Py_END_ALLOW_THREADS
 
-	if (ret) {
+	if (ret < 0) {
 		PyUSB_Error();
 		return NULL;
 	} else {
@@ -1666,7 +1694,7 @@ PYUSB_STATIC PyObject *Py_usb_DeviceHandle_clearHalt(
 	ret = usb_clear_halt(_self->deviceHandle, endpoint);
 	Py_END_ALLOW_THREADS
 
-	if (ret) {
+	if (ret < 0) {
 		PyUSB_Error();
 		return NULL;
 	} else {
